@@ -5,13 +5,12 @@
 # This software may be used and distributed according to the terms of the
 # GNU General Public License version 2 or any later version.
 
-from __future__ import absolute_import
 
+from typing import Any, List, Optional, Tuple
 from . import (
     configitems,
     error,
     pycompat,
-    util,
 )
 
 # unlike the other registered items, config options are neither functions or
@@ -21,7 +20,8 @@ from . import (
 # the other items extensions want might to register.
 configitem = configitems.getitemregister
 
-class _funcregistrarbase(object):
+
+class _funcregistrarbase:
     """Base of decorator to register a function for specific purpose
 
     This decorator stores decorated functions into own dict 'table'.
@@ -36,7 +36,7 @@ class _funcregistrarbase(object):
 
         keyword = registrar.keyword()
 
-        @keyword('bar')
+        @keyword(b'bar')
         def barfunc(*args, **kwargs):
             '''Explanation of bar keyword ....
             '''
@@ -47,6 +47,7 @@ class _funcregistrarbase(object):
     - 'barfunc' is stored as 'bar' in '_table' of an instance 'keyword' above
     - 'barfunc.__doc__' becomes ":bar: Explanation of bar keyword"
     """
+
     def __init__(self, table=None):
         if table is None:
             self._table = {}
@@ -60,12 +61,12 @@ class _funcregistrarbase(object):
         name = self._getname(decl)
 
         if name in self._table:
-            msg = 'duplicate registration for name: "%s"' % name
+            msg = b'duplicate registration for name: "%s"' % name
             raise error.ProgrammingError(msg)
 
-        if func.__doc__ and not util.safehasattr(func, '_origdoc'):
-            doc = pycompat.sysbytes(func.__doc__).strip()
-            func._origdoc = doc
+        if func.__doc__ and not hasattr(func, '_origdoc'):
+            func._origdoc = func.__doc__.strip()
+            doc = pycompat.sysbytes(func._origdoc)
             func.__doc__ = pycompat.sysstr(self._formatdoc(decl, doc))
 
         self._table[name] = func
@@ -73,10 +74,28 @@ class _funcregistrarbase(object):
 
         return func
 
-    def _parsefuncdecl(self, decl):
-        """Parse function declaration and return the name of function in it
+    def _merge(self, registrarbase):
+        """Merge the entries of the given registrar object into this one.
+
+        The other registrar object must not contain any entries already in the
+        current one, or a ProgrammmingError is raised.  Additionally, the types
+        of the two registrars must match.
         """
-        i = decl.find('(')
+        if not isinstance(registrarbase, type(self)):
+            msg = b"cannot merge different types of registrar"
+            raise error.ProgrammingError(msg)
+
+        dups = set(registrarbase._table).intersection(self._table)
+
+        if dups:
+            msg = b'duplicate registration for names: "%s"' % b'", "'.join(dups)
+            raise error.ProgrammingError(msg)
+
+        self._table.update(registrarbase._table)
+
+    def _parsefuncdecl(self, decl):
+        """Parse function declaration and return the name of function in it"""
+        i = decl.find(b'(')
         if i >= 0:
             return decl[:i]
         else:
@@ -100,8 +119,8 @@ class _funcregistrarbase(object):
         return self._docformat % (decl, doc)
 
     def _extrasetup(self, name, func):
-        """Execute exra setup for registered function, if needed
-        """
+        """Execute extra setup for registered function, if needed"""
+
 
 class command(_funcregistrarbase):
     """Decorator to register a command function to table
@@ -169,23 +188,37 @@ class command(_funcregistrarbase):
     """
 
     # Command categories for grouping them in help output.
-    CATEGORY_REPO_CREATION = 'repo'
-    CATEGORY_REMOTE_REPO_MANAGEMENT = 'remote'
-    CATEGORY_COMMITTING = 'commit'
-    CATEGORY_CHANGE_MANAGEMENT = 'management'
-    CATEGORY_CHANGE_ORGANIZATION = 'organization'
-    CATEGORY_FILE_CONTENTS = 'files'
-    CATEGORY_CHANGE_NAVIGATION  = 'navigation'
-    CATEGORY_WORKING_DIRECTORY = 'wdir'
-    CATEGORY_IMPORT_EXPORT = 'import'
-    CATEGORY_MAINTENANCE = 'maintenance'
-    CATEGORY_HELP = 'help'
-    CATEGORY_MISC = 'misc'
-    CATEGORY_NONE = 'none'
+    # These can also be specified for aliases, like:
+    # [alias]
+    # myalias = something
+    # myalias:category = repo
+    CATEGORY_REPO_CREATION = b'repo'
+    CATEGORY_REMOTE_REPO_MANAGEMENT = b'remote'
+    CATEGORY_COMMITTING = b'commit'
+    CATEGORY_CHANGE_MANAGEMENT = b'management'
+    CATEGORY_CHANGE_ORGANIZATION = b'organization'
+    CATEGORY_FILE_CONTENTS = b'files'
+    CATEGORY_CHANGE_NAVIGATION = b'navigation'
+    CATEGORY_WORKING_DIRECTORY = b'wdir'
+    CATEGORY_IMPORT_EXPORT = b'import'
+    CATEGORY_MAINTENANCE = b'maintenance'
+    CATEGORY_HELP = b'help'
+    CATEGORY_MISC = b'misc'
+    CATEGORY_NONE = b'none'
 
-    def _doregister(self, func, name, options=(), synopsis=None,
-                    norepo=False, optionalrepo=False, inferrepo=False,
-                    intents=None, helpcategory=None, helpbasic=False):
+    def _doregister(
+        self,
+        func,
+        name,
+        options=(),
+        synopsis=None,
+        norepo=False,
+        optionalrepo=False,
+        inferrepo=False,
+        intents=None,
+        helpcategory=None,
+        helpbasic=False,
+    ):
         func.norepo = norepo
         func.optionalrepo = optionalrepo
         func.inferrepo = inferrepo
@@ -198,8 +231,16 @@ class command(_funcregistrarbase):
             self._table[name] = func, list(options)
         return func
 
+    def rename(self, old, new):
+        """rename a command. Used to add aliases, debugstrip ->
+        debugstrip|strip
+        """
+        self._table[new] = self._table.pop(old)
+
+
 INTENT_READONLY = b'readonly'
 
+
 class revsetpredicate(_funcregistrarbase):
     """Decorator to register revset predicate
 
@@ -207,7 +248,7 @@ class revsetpredicate(_funcregistrarbase):
 
         revsetpredicate = registrar.revsetpredicate()
 
-        @revsetpredicate('mypredicate(arg1, arg2[, arg3])')
+        @revsetpredicate(b'mypredicate(arg1, arg2[, arg3])')
         def mypredicatefunc(repo, subset, x):
             '''Explanation of this revset predicate ....
             '''
@@ -240,14 +281,16 @@ class revsetpredicate(_funcregistrarbase):
 
     Otherwise, explicit 'revset.loadpredicate()' is needed.
     """
+
     _getname = _funcregistrarbase._parsefuncdecl
-    _docformat = "``%s``\n    %s"
+    _docformat = b"``%s``\n    %s"
 
     def _extrasetup(self, name, func, safe=False, takeorder=False, weight=1):
         func._safe = safe
         func._takeorder = takeorder
         func._weight = weight
 
+
 class filesetpredicate(_funcregistrarbase):
     """Decorator to register fileset predicate
 
@@ -255,7 +298,7 @@ class filesetpredicate(_funcregistrarbase):
 
         filesetpredicate = registrar.filesetpredicate()
 
-        @filesetpredicate('mypredicate()')
+        @filesetpredicate(b'mypredicate()')
         def mypredicatefunc(mctx, x):
             '''Explanation of this fileset predicate ....
             '''
@@ -289,17 +332,20 @@ class filesetpredicate(_funcregistrarbase):
 
     Otherwise, explicit 'fileset.loadpredicate()' is needed.
     """
+
     _getname = _funcregistrarbase._parsefuncdecl
-    _docformat = "``%s``\n    %s"
+    _docformat = b"``%s``\n    %s"
 
     def _extrasetup(self, name, func, callstatus=False, weight=1):
         func._callstatus = callstatus
         func._weight = weight
 
+
 class _templateregistrarbase(_funcregistrarbase):
-    """Base of decorator to register functions as template specific one
-    """
-    _docformat = ":%s: %s"
+    """Base of decorator to register functions as template specific one"""
+
+    _docformat = b":%s: %s"
+
 
 class templatekeyword(_templateregistrarbase):
     """Decorator to register template keyword
@@ -309,25 +355,16 @@ class templatekeyword(_templateregistrarbase):
         templatekeyword = registrar.templatekeyword()
 
         # new API (since Mercurial 4.6)
-        @templatekeyword('mykeyword', requires={'repo', 'ctx'})
+        @templatekeyword(b'mykeyword', requires={b'repo', b'ctx'})
         def mykeywordfunc(context, mapping):
             '''Explanation of this template keyword ....
             '''
             pass
 
-        # old API (DEPRECATED)
-        @templatekeyword('mykeyword')
-        def mykeywordfunc(repo, ctx, templ, cache, revcache, **args):
-            '''Explanation of this template keyword ....
-            '''
-            pass
-
     The first string argument is used also in online help.
 
     Optional argument 'requires' should be a collection of resource names
-    which the template keyword depends on. This also serves as a flag to
-    switch to the new API. If 'requires' is unspecified, all template
-    keywords and resources are expanded to the function arguments.
+    which the template keyword depends on.
 
     'templatekeyword' instance in example above can be used to
     decorate multiple functions.
@@ -339,9 +376,10 @@ class templatekeyword(_templateregistrarbase):
     Otherwise, explicit 'templatekw.loadkeyword()' is needed.
     """
 
-    def _extrasetup(self, name, func, requires=None):
+    def _extrasetup(self, name, func, requires=()):
         func._requires = requires
 
+
 class templatefilter(_templateregistrarbase):
     """Decorator to register template filer
 
@@ -349,7 +387,7 @@ class templatefilter(_templateregistrarbase):
 
         templatefilter = registrar.templatefilter()
 
-        @templatefilter('myfilter', intype=bytes)
+        @templatefilter(b'myfilter', intype=bytes)
         def myfilterfunc(text):
             '''Explanation of this template filter ....
             '''
@@ -373,6 +411,7 @@ class templatefilter(_templateregistrarbase):
     def _extrasetup(self, name, func, intype=None):
         func._intype = intype
 
+
 class templatefunc(_templateregistrarbase):
     """Decorator to register template function
 
@@ -380,8 +419,8 @@ class templatefunc(_templateregistrarbase):
 
         templatefunc = registrar.templatefunc()
 
-        @templatefunc('myfunc(arg1, arg2[, arg3])', argspec='arg1 arg2 arg3',
-                      requires={'ctx'})
+        @templatefunc(b'myfunc(arg1, arg2[, arg3])', argspec=b'arg1 arg2 arg3',
+                      requires={b'ctx'})
         def myfuncfunc(context, mapping, args):
             '''Explanation of this template function ....
             '''
@@ -405,12 +444,14 @@ class templatefunc(_templateregistrarbase):
 
     Otherwise, explicit 'templatefuncs.loadfunction()' is needed.
     """
+
     _getname = _funcregistrarbase._parsefuncdecl
 
     def _extrasetup(self, name, func, argspec=None, requires=()):
         func._argspec = argspec
         func._requires = requires
 
+
 class internalmerge(_funcregistrarbase):
     """Decorator to register in-process merge tool
 
@@ -418,7 +459,7 @@ class internalmerge(_funcregistrarbase):
 
         internalmerge = registrar.internalmerge()
 
-        @internalmerge('mymerge', internalmerge.mergeonly,
+        @internalmerge(b'mymerge', internalmerge.mergeonly,
                        onfailure=None, precheck=None,
                        binary=False, symlink=False):
         def mymergefunc(repo, mynode, orig, fcd, fco, fca,
@@ -466,16 +507,24 @@ class internalmerge(_funcregistrarbase):
 
     Otherwise, explicit 'filemerge.loadinternalmerge()' is needed.
     """
-    _docformat = "``:%s``\n    %s"
+
+    _docformat = b"``:%s``\n    %s"
 
     # merge type definitions:
     nomerge = None
-    mergeonly = 'mergeonly'  # just the full merge, no premerge
-    fullmerge = 'fullmerge'  # both premerge and merge
+    mergeonly = b'mergeonly'  # just the full merge, no premerge
+    fullmerge = b'fullmerge'  # both premerge and merge
 
-    def _extrasetup(self, name, func, mergetype,
-                    onfailure=None, precheck=None,
-                    binary=False, symlink=False):
+    def _extrasetup(
+        self,
+        name,
+        func,
+        mergetype,
+        onfailure=None,
+        precheck=None,
+        binary=False,
+        symlink=False,
+    ):
         func.mergetype = mergetype
         func.onfailure = onfailure
         func.precheck = precheck
@@ -484,4 +533,31 @@ class internalmerge(_funcregistrarbase):
         symlinkcap = symlink or mergetype == self.nomerge
 
         # actual capabilities, which this internal merge tool has
-        func.capabilities = {"binary": binarycap, "symlink": symlinkcap}
+        func.capabilities = {b"binary": binarycap, b"symlink": symlinkcap}
+
+
+class verify_check(_funcregistrarbase):
+    """Decorator to register a check for admin::verify
+
+    options is a list of (name, default value, help) to be passed to the check
+    """
+
+    def __init__(self, table=None, alias_table=None):
+        super().__init__(table)
+        if alias_table is None:
+            self._alias_table = {}
+        else:
+            self._alias_table = alias_table
+
+    def _extrasetup(
+        self,
+        name,
+        func,
+        alias: Optional[bytes] = None,
+        options: Optional[List[Tuple[bytes, Any, bytes]]] = None,
+    ):
+        func.alias = alias
+        func.options = options
+
+        if alias:
+            self._alias_table[alias] = name
